require_relative 'spec_helper'

describe Api::Handler do
  let(:user) { {login: 'login', password: 'password'}  }
  let(:app) { {api_key: 'api_key'} }
  let(:opt) { {user: true, app: false} }
  let(:token) { 'a token' }
  let(:session_info) { {login: 'login', token: token} }
   nb = rand(1000)
  let(:a_user) {{model_name: 'User',content: {login: 'macoumba'+nb.to_s,password: 'passeraa',firstname: 'Ndeye Fatou', lastname: 'Dieng',email: 'macoumba'+nb.to_s+'@gmail.com',gender: 'M'}} }
  let(:param_for_read) { {model_name: 'User', condition: {login: a_user[:content][:login]}} }

  describe '.connect' do
    before(:each) do
      AuthManager::Base.redis_del({list: 'users'})
      AuthManager::Base.redis_del({list: 'applications'})
      AuthManager::Base.redis_sadd({list: 'users', object: user})
      AuthManager::Base.redis_sadd({list: 'applications', object: app})
    end
    context 'When user connected' do
      it {
        swap_option_to('user')
        expect(Api::Handler.connect(user.to_json, opt)[:code]).to eq(200)
      }
    end
    context 'When app connected' do
      it {
        swap_option_to('app')
        app[:api_key] = 'api_key'
        expect(Api::Handler.connect(app.to_json, opt)[:code]).to eq(200)
      }
    end
    context 'When user fails to connect' do
      it {
        swap_option_to('user')
        user[:login] = 'not a login'
        expect(Api::Handler.connect(user.to_json, opt)[:code]).to eq(400)
      }
    end
    context 'When app fails to connect' do
      it {
        swap_option_to('app')
        app[:api_key] = 'not an api_key'
        expect(Api::Handler.connect(app.to_json, opt)[:code]).to eq(400)
      }
    end
  end
  describe '.create_object' do
    context 'When user account created' do
      it {
        expect(Api::Handler.create_object(a_user)).to be_truthy
      }
    end
    context 'When user account not created' do
      it {
        expect(Api::Handler.create_object(a_user)).to be_falsey
      }
    end
  end
  describe '.valid_session' do
    before(:each) do
      AuthManager::Base.redis_del({list: 'users'})
      AuthManager::Base.redis_del({list: 'applications_sessions'})
      AuthManager::Base.redis_del({list: 'users_sessions'})
      AuthManager::Base.redis_sadd({list: 'users', object: user })
      AuthManager::Base.redis_sadd({list: 'users_sessions', object: session_info })
    end
    context 'When session is active' do
      it {
        swap_option_to('user')
        user[:login] = 'login'
        expect(Api::Handler.valid_session(user.to_json, token, opt)).to be_truthy
      }
    end
    context 'When session is not active' do
      it {
        swap_option_to('user')
        user[:login] = 'not a login'
        expect(Api::Handler.valid_session(user.to_json, token, opt)).to be_falsey
      }
    end
  end

  describe '.condition_read' do
    context 'Record found' do
      it { expect(Api::Handler.condition_read(param_for_read)[:code]).to eq (200)}
    end
    context 'Record not found' do
      it do
        param_for_read[:condition][:login] = 'not a login'
        expect(Api::Handler.condition_read(param_for_read)[:code]).to eq(404)
      end
    end
  end
  #to do later
  describe '.request' do

  end
end
